/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.datamodels;

/**
 * Enumeration of possible End of Life States: working, repairable, broken.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public enum WEEEsEndOfLifeState {
	/**
	 * The device is still in working order.
	 */
	WORKING("working", 1.0),
	/**
	 * The device can be repaired and reused.
	 */
	REUSABLE("repairable", 0.5),
	/**
	 * The device is broken and is feasible to recover its materials.
	 */
	BROKEN("broken", 0.0);

	/**
	 * End of Life {@link String} representation.
	 */
	private final String m_name;

	/**
	 * End of Life value of the device state.
	 */
	private final double m_value;

	/**
	 * Constructor.
	 * 
	 * @param name  the End of Life State name.
	 * @param value the End of Life State value.
	 */
	private WEEEsEndOfLifeState(String name, double value) {
		m_name = name;
		m_value = value;
	}

	@Override
	public String toString() {
		return m_name;
	}

	/**
	 * @return the corresponding value of the End of Life state.
	 */
	public double Value() {
		return m_value;
	}
}
