/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.logging;

import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;

import gr.iccs.isense.circ4life.rest.config.ConfigProperties;

/**
 * Common logging facilities.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class Log {
	/**
	 * Creates or returns the existing logger for a specified class.
	 * 
	 * @param <T>   the type of the class modeled by the specified
	 *              <code>clazz</code> object.
	 * @param clazz the class to get the logger for.
	 * @return the logger for the specified class, creating it on demand.
	 */
	public static <T> Logger getConfiguredLogger(final Class<T> clazz) {
		final Logger logger = Logger.getLogger("" + clazz);

		// Get the configuration properties and set them on the logger and its new
		// additional handler
		logger.setLevel(ConfigProperties.getLoggingLevel(clazz));
		final Handler handler = getDefaultLoggerHandler();
		handler.setLevel(logger.getLevel());
		// This will add our default logger handler to the list of existing ones, which
		// may be empty.
		logger.addHandler(handler);
		// Do not inherit the parent loggers handlers
		logger.setUseParentHandlers(false);

		logger.info(clazz.getSimpleName() + " Logger level: " + logger.getLevel() + ", Verbosity: "
				+ ConfigProperties.getLoggingVerbosity(clazz) + ", Parent Logger Level: "
				+ logger.getParent().getLevel() + ", using parent logger handlers: " + logger.getUseParentHandlers());

		return logger;
	}

	/**
	 * Create and return the default logger handler.
	 * 
	 * @return a new default logger handler.
	 */
	private static Handler getDefaultLoggerHandler() {
		return new ConsoleHandler();
	}
}
