/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest.resources;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Abstract REST end-point for Web Service information. Implements all
 * functionality except the version string which is the responsibility of each
 * service to do in a subclass.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public abstract class AbstractInformation {

	/**
	 * @return the version of this Web Service as it is saved in MANIFEST from
	 *         attribute 'Implementation-Version'.
	 */
	protected abstract String getVersion();

	/**
	 * @return the version of this Web Service as text/plain.
	 */
	@GET
	@Path("version")
	@Produces(MediaType.TEXT_PLAIN)
	public Response versionTextPlain() {
		final String version = getVersion();

		return Response.ok(version, MediaType.TEXT_PLAIN).build();
	}

	/**
	 * @return the version of this Web Service as text/html.
	 */
	@GET
	@Path("version")
	@Produces(MediaType.TEXT_HTML)
	public Response versionTextHTML() {
		final String version = getVersion();

		return Response.ok("<!doctype html>" + "<html lang=en>" + "<head>" + "<meta charset=utf-8>" + "</head>"
				+ "<body>" + "<p>" + version + "</p>" + "</body>" + "</html>", MediaType.TEXT_HTML).build();
	}

	/**
	 * @return the version of this Web Service as application/xml.
	 */
	@GET
	@Path("version")
	@Produces(MediaType.APPLICATION_XML)
	public Response versionApplicationXML() {
		final String version = getVersion();

		return Response.ok("<?xml version=\"1.0\" encoding=\"UTF-8\"?><version>" + version + "</version>",
				MediaType.APPLICATION_XML).build();
	}

	/**
	 * @return the version of this Web Service as application/json.
	 */
	@GET
	@Path("version")
	@Produces(MediaType.APPLICATION_JSON)
	public Response versionApplicationJSON() {
		final String version = getVersion();

		return Response.ok("{ \"version\": \"" + version + "\"}", MediaType.APPLICATION_JSON).build();
	}
}
