/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.servlet;

import java.io.IOException;
import java.util.logging.Logger;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.keycloak.adapters.servlet.KeycloakOIDCFilter;

import gr.iccs.isense.circ4life.logging.Log;
import gr.iccs.isense.circ4life.rest.config.ConfigProperties;

public class AccessControlManagerFilter extends KeycloakOIDCFilter {
	/**
	 * Logger.
	 */
	public final static Logger logger = Log.getConfiguredLogger(AccessControlManagerFilter.class);

	/**
	 * Flag that denotes if the Access Control Manager is enabled for this REST
	 * Service or not.
	 */
	private static boolean AccessControlManagerEnabled = false;

	/**
	 * Constructor.
	 */
	public AccessControlManagerFilter() {
		AccessControlManagerEnabled = ConfigProperties.isAccessControlManagerEnabled(AccessControlManagerFilter.class);

		logger.info("Access Control Manager enabled: " + AccessControlManagerEnabled);

		// Add also the handler to the super class logger
		final Logger superclasslogger = Log.getConfiguredLogger(KeycloakOIDCFilter.class);
		superclasslogger.info("Keycloak OpenID Connect Filter enabled: " + AccessControlManagerEnabled);
	}

	/**
	 * @return <code>true</code> if the Access Control Manager is enabled,
	 *         <code>false</code> otherwise.
	 */
	public static boolean isAccessControlManagerEnabled() {
		return AccessControlManagerEnabled;
	}

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		final String cfgpath = filterConfig.getInitParameter(CONFIG_PATH_PARAM);
		logger.info("Access Control Manager configuration path: " + cfgpath);
		super.init(filterConfig);
	}

	@Override
	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain)
			throws IOException, ServletException {
		logger.finest(AccessControlManagerFilter.class + " doFilter");
		if (AccessControlManagerEnabled) {
			// Process the filter from the parent class to apply access control
			logger.fine(AccessControlManagerFilter.class + "Access Control Manager is enabled.");
			super.doFilter(req, res, chain);
			return;
		} else {
			logger.warning(AccessControlManagerFilter.class + "Access Control Manager is disabled.");
			// Go to the next filter in the chain, skipping access control
			chain.doFilter(req, res);
			return;
		}
	}
}
