/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.validation;

import java.util.logging.Logger;

import gr.iccs.isense.circ4life.logging.Log;

/**
 * User resignation form validator.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class EndUserResignationFormValidator {
	/**
	 * Logger.
	 */
	public final static Logger logger = Log.getConfiguredLogger(EndUserResignationFormValidator.class);

	/**
	 * Check the validity of parameters.
	 * 
	 * @param email    e-mail.
	 * @return <code>true</code> if all parameters are valid, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isValid(final String email) {
		try {
			// Email validation
			if (null == email || !EmailAddressValidator.isValid(email)) {
				throw new IllegalArgumentException("Illegal user e-mail: " + email);
			}
		} catch (Exception e) {
			logger.severe("Invalid resignation form, details: " + e.getMessage());

			return false;
		}

		// All OK
		return true;
	}
}
