/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.test;

import java.net.URI;
import java.util.regex.Pattern;

import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.grizzly.servlet.FilterRegistration;
import org.glassfish.grizzly.servlet.ServletRegistration;
import org.glassfish.grizzly.servlet.WebappContext;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpServerFactory;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.servlet.ServletContainer;

import gr.iccs.isense.circ4life.servlet.AccessControlManagerFilter;

/**
 * Standalone Application Server for local launching of the REST Web Services.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class ApplicationServer {
	private static final String ACCCTRLMNGRFILTERNAME = "Keycloak OpenID Connect Filter";
	private static final String ACCCTRLMNGRFILTERINITPARAMSKIP = "keycloak.config.skipPattern";
	private static final String ACCCTRLMNGRFILTERINITPARAMKEYCLOAKCFGPATH = "keycloak.config.path";
	private static final String KEYCLOAKCFGPATH = "/keycloak.json";

	public static HttpServer ServerInstance = null;

	/**
	 * Program entry point.
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String[] args) {
		if (4 != args.length || null == args[0] || null == args[1] || null == args[2] || null == args[3]
				|| !Pattern.matches("http://127.0.0.1:\\d+/.*", args[2])) {
			System.err.println("Usage: " + ApplicationServer.class.getName()
					+ " DISPLAYNAME APPCLASSNAME LOCALHOST_URI ACCESSCONTROLSKIPPATTERN");
			System.err.println(
					"\tDISPLAYNAME: The Web Application context display name (web.xml display-name tag equivalent)");
			System.err.println(
					"\tAPPCLASSNAME: The full application class name, e.g. gr.iccs.isense.circ4life.rest.EcoCreditCalculatorModule");
			System.err.println(
					"\tLOCALHOST_URI: The locahost URI to listen to with pattern: http://127.0.0.1:PORT/CONTEXTPATH/RESOURCEPATH");
			System.err.println("\tACCESSCONTROLSKIPPATTERN: URI path pattern to skip access control for.");
			System.exit(1);
		}

		System.out.println("Starting debugging application server ...");
		System.out.println("DISPLAYNAME: " + args[0]);
		System.out.println("APPCLASSNAME: " + args[1]);
		System.out.println("URI: " + args[2]);
		System.out.println("ACCESSCONTROLSKIPPATTERN: " + args[3]);

		try {
			final String DisplayName = args[0];
			final Class<?> AppClass = Class.forName(args[1]);
			final URI uri = new URI(args[2]);
			final String accessControlSkipPattern = args[3];

			// Get the context path for the resources by removing scheme, host, port and
			// ending /
			final String uriPath = uri.getPath().replaceAll("/{2,}", "/").replaceAll("^/", "").replaceAll("/$", "");
			final String[] tokens = uriPath.split("/");
			final String contextPath = "/" + tokens[0];
			final String urlPatterns = "/" + tokens[1] + "/*";

			System.out.println("Context path: " + contextPath);
			System.out.println("URL patterns: " + urlPatterns);

			// Configure the Web app as in web.xml
			final WebappContext webappcxt = new WebappContext(DisplayName, contextPath);
			final ServletContainer servletContainer = new ServletContainer((ResourceConfig) AppClass.newInstance());
			final ServletRegistration servletReg = webappcxt.addServlet(AppClass.getName(), servletContainer);
			servletReg.addMapping(urlPatterns);
			final FilterRegistration filterReg = webappcxt.addFilter(ACCCTRLMNGRFILTERNAME,
					AccessControlManagerFilter.class);
			filterReg.setInitParameter(ACCCTRLMNGRFILTERINITPARAMKEYCLOAKCFGPATH, KEYCLOAKCFGPATH);
			filterReg.setInitParameter(ACCCTRLMNGRFILTERINITPARAMSKIP, accessControlSkipPattern);
			filterReg.addMappingForUrlPatterns(null, urlPatterns);

			// Create the server
			ServerInstance = GrizzlyHttpServerFactory.createHttpServer(uri, false);

			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
				@Override
				public void run() {
					ServerInstance.shutdownNow();
				}
			}));

			// Deploy and start the server
			webappcxt.deploy(ServerInstance);
			ServerInstance.start();

			System.out.println("Started debugging server, URL " + uri + ".");

			Thread.currentThread().join();
		} catch (Exception e) {
			System.err.println("Could not start server, details: " + e.getMessage());
			System.err.println("Stacktrace:");
			final StackTraceElement stacktrace[] = e.getStackTrace();
			for (final StackTraceElement st : stacktrace) {
				System.err.println(st.toString());
			}
			System.exit(1);
		}
	}
}
