/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.test;

import gr.iccs.isense.circ4life.datamodels.MaterialFactors;

/**
 * Common test objects.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class TestsCommon {
	/**
	 * Maximum accepted difference on comparison of double floating point fractional
	 * values. If the difference is less than that, the result can be considered a
	 * valid.
	 */
	public static final double fractional_max_accepted_delta = 0.00000001;

	/**
	 * Maximum accepted difference on comparison of double floating point integer
	 * values. If the difference is less than that, the result can be considered a
	 * valid.
	 */
	public static final double integer_max_accepted_delta = 0.2;

	/**
	 * Example material factors for a working tablet.
	 */
	public static final MaterialFactors working_tablet_material_factors[] = {
			// Material factor, recyclability = 1.0
			new MaterialFactors(0.02175, 8398.0, 1.0), // Silver
			new MaterialFactors(48.41617, 681.7, 1.0), // Aluminum
			new MaterialFactors(0.02245, 428.0, 1.0), // Arsenic
			new MaterialFactors(0.01477, 663306.6, 1.0), // Gold
			new MaterialFactors(0.00021, 64441.0, 1.0), // Cadmium
			new MaterialFactors(0.07586, 41.0, 1.0), // Chromium
			new MaterialFactors(5.55894, 348.0, 1.0), // Copper
			new MaterialFactors(2.31970, 32.0, 1.0), // Iron
			new MaterialFactors(0.00007, 28707.0, 1.0), // Mercury
			new MaterialFactors(0.01816, 363917.0, 1.0), // Indium
			new MaterialFactors(0.28112, 583.0, 1.0), // Magnesite
			new MaterialFactors(0.37310, 73.0, 1.0), // Manganese
			new MaterialFactors(0.36155, 877.0, 1.0), // Nickel
			new MaterialFactors(0.54578, 41.0, 1.0), // Lead
			new MaterialFactors(0.00009, 2870013.0, 1.0), // Palladium
			new MaterialFactors(0.07044, 488.0, 1.0), // Antimony
			new MaterialFactors(62.02000, 1.45, 1.0), // Silicon
			new MaterialFactors(0.91551, 732.0, 1.0), // Tin
			new MaterialFactors(0.18655, 140.0, 1.0), // Titanium
			new MaterialFactors(0.61363, 1683.0, 1.0), // Zinc
			new MaterialFactors(22.50000, 26.75, 1.0), // PMMA
			new MaterialFactors(165.00000, 39.84, 1.0) // ABS
	};

	/**
	 * Example material factors for a broken tablet.
	 */
	public static final MaterialFactors broken_tablet_material_factors[] = {
			// Material factor, recyclability values depend on material
			new MaterialFactors(0.02175, 8398.0, 0.5), // Silver
			new MaterialFactors(48.41617, 681.7, 0.85), // Aluminum
			new MaterialFactors(0.02245, 428.0, 0.0), // Arsenic
			new MaterialFactors(0.01477, 663306.6, 0.5), // Gold
			new MaterialFactors(0.00021, 64441.0, 0.0), // Cadmium
			new MaterialFactors(0.07586, 41.0, 0.0), // Chromium
			new MaterialFactors(5.55894, 348.0, 1.0), // Copper
			new MaterialFactors(2.31970, 32.0, 1.0), // Iron
			new MaterialFactors(0.00007, 28707.0, 0.0), // Mercury
			new MaterialFactors(0.01816, 363917.0, 0.0), // Indium
			new MaterialFactors(0.28112, 583.0, 0.0), // Magnesite
			new MaterialFactors(0.37310, 73.0, 0.0), // Manganese
			new MaterialFactors(0.36155, 877.0, 0.0), // Nickel
			new MaterialFactors(0.54578, 41.0, 0.0), // Lead
			new MaterialFactors(0.00009, 2870013.0, 0.5), // Palladium
			new MaterialFactors(0.07044, 488.0, 0.0), // Antimony
			new MaterialFactors(62.02000, 1.45, 0.0), // Silicon
			new MaterialFactors(0.91551, 732.0, 0.0), // Tin
			new MaterialFactors(0.18655, 140.0, 0.0), // Titanium
			new MaterialFactors(0.61363, 1683.0, 0.5), // Zinc
			new MaterialFactors(22.50000, 26.75, 1.0), // PMMA
			new MaterialFactors(165.00000, 39.84, 0.95) // ABS
	};

	/**
	 * Product type test parameters encapsulation.
	 */
	public final static class EcoCreditCalculatorProductTypeTest {
		/**
		 * The expected Eco-credits result.
		 */
		public final double expectedresult;

		/**
		 * Product Harmonized System code.
		 */
		public final int hskey;

		/**
		 * Years of product usage.
		 */
		public final int years;

		/**
		 * Extra months of usage added to years.
		 */
		public final int months;

		/**
		 * The End of Life state.
		 */
		public final String eolstate;

		/**
		 * Constructor.
		 * 
		 * @param expectedresult The expected Eco-credits result.
		 * @param hskey          Product Harmonized System code.
		 * @param years          years of product usage.
		 * @param months         extra months of usage added to years.
		 * @param eolstate       the End of Life state.
		 */
		public EcoCreditCalculatorProductTypeTest(double expectedresult, int hskey, int years, int months,
				String eolstate) {
			this.expectedresult = expectedresult;
			this.hskey = hskey;
			this.years = years;
			this.months = months;
			this.eolstate = eolstate;
		}
	}

	/**
	 * Bio waste test parameters encapsulation.
	 */
	public final static class EcoCreditCalculatorBioWasteTest {
		/**
		 * The expected Eco-credits result.
		 */
		public final double expectedresult;

		/**
		 * Organic waste mass.
		 */
		public final double waste_mass;

		/**
		 * Constructor.
		 * 
		 * @param expectedresult the expected Eco-credits result.
		 * @param waste_mass     the waste mass in grams.
		 */
		public EcoCreditCalculatorBioWasteTest(double expectedresult, double waste_mass) {
			this.expectedresult = expectedresult;
			this.waste_mass = waste_mass;
		}
	}
}