/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.integration.test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import org.junit.Assert;
import org.junit.Test;

import gr.iccs.isense.circ4life.dao.CommonDAO.EcoAccountRecord;
import gr.iccs.isense.circ4life.dao.CommonDAO.IndividualProductData;
import gr.iccs.isense.circ4life.dao.CommonDAO.ProductID;
import gr.iccs.isense.circ4life.dao.CommonDAO.ProductType;
import gr.iccs.isense.circ4life.dao.CommonDAO.SystemFeedback;
import gr.iccs.isense.circ4life.dao.CommonDAO.UserProductFeedback;
import gr.iccs.isense.circ4life.datamodels.WEEEsEndOfLifeState;
import gr.iccs.isense.circ4life.rest.resources.AccessControlManager;
import gr.iccs.isense.circ4life.rest.resources.openapi.DataConversion;
import gr.iccs.isense.circ4life.test.IntegrationTestsCommon.TestUser;

/**
 * Integration tests for the Data Access Object (DAO) for Eco-account.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class EcoAccountDAOIT extends PersistenceLayerDAOIT {
	@Test
	public void itEcoAccountLocale() throws IllegalArgumentException, MalformedURLException {
		final String[] locales = { "nl", // -> Dutch
				"fr", // -> French
				"fi", // -> Finnish
				"de", // -> German
				"pl", // -> Polish
				"es", // -> Spanish
				"sv", // -> Swedish
				"en" // -> English
		};

		for (String locale : locales) {
			final String useruuid = ecoaccountdao.createEcoAccount(locale + TestUser.Username, TestUser.Password,
					locale + TestUser.Email, TestUser.Firstname, TestUser.Lastname,
					DataConversion.readDate(TestUser.birthday), TestUser.country, TestUser.postalcode, TestUser.street,
					TestUser.city, TestUser.region, TestUser.telephone, new URL(TestUser.image), new URL(TestUser.url),
					TestUser.gender, TestUser.organizationname, new URL(TestUser.organizationURL), locale);

			Assert.assertNotNull(useruuid);

			Assert.assertTrue(ecoaccountdao.updateEcoAccount(useruuid, "87654321", null, null, null, null, null, null, null,
					null, null, null, null, null, null, null, null, null));
			Assert.assertTrue(ecoaccountdao.updateEcoAccount(useruuid, null, null, null, null, null, "Greece", null, null,
					null, null, null, null, null, null, null, null, null));

			Assert.assertTrue(ecoaccountdao.removeEcoAccount(useruuid));
		}
	}

	@Test
	public void itEcoAccountAccess() {
		final String useruuid = AccessControlManager.getUsernameUUID(TestUser.Username);

		final EcoAccountRecord account = ecoaccountdao.getEcoAccount(useruuid);
		Assert.assertNotNull(account);
		Assert.assertTrue(useruuid.contentEquals(account.getEcoAccount().getIdentifier()));
		Assert.assertTrue(TestUser.Email.contentEquals(account.getEcoAccount().getEmail()));
		Assert.assertTrue(TestUser.Firstname.contentEquals(account.getEcoAccount().getGivenName()));
		Assert.assertTrue(TestUser.Lastname.contentEquals(account.getEcoAccount().getFamilyName()));
		Assert.assertTrue(TestUser.birthday.contentEquals(account.getEcoAccount().getBirthDate()));
		Assert.assertTrue(TestUser.country.contentEquals(account.getEcoAccount().getAddress().getAddressCountry()));
		Assert.assertTrue(TestUser.postalcode.contentEquals(account.getEcoAccount().getAddress().getPostalCode()));
		Assert.assertTrue(TestUser.street.contentEquals(account.getEcoAccount().getAddress().getStreetAddress()));
		Assert.assertTrue(TestUser.city.contentEquals(account.getEcoAccount().getAddress().getAddressLocality()));
		Assert.assertTrue(TestUser.region.contentEquals(account.getEcoAccount().getAddress().getAddressRegion()));
		Assert.assertTrue(TestUser.telephone.contentEquals(account.getEcoAccount().getTelephone()));
		Assert.assertTrue(TestUser.image.contentEquals(account.getEcoAccount().getImage()));
		Assert.assertTrue(TestUser.url.contentEquals(account.getEcoAccount().getUrl()));
		Assert.assertTrue(TestUser.gender.contentEquals(account.getEcoAccount().getGender()));
		Assert.assertTrue(TestUser.organizationname.contentEquals(account.getEcoAccount().getAffiliation().getName()));
		Assert.assertTrue(TestUser.organizationURL.contentEquals(account.getEcoAccount().getAffiliation().getUrl()));
		Assert.assertTrue(TestUser.recyclebinuserid == account.getEcoBalance().getRecycleBinUserID().doubleValue());

		final String anonymousid = ecoaccountdao.getEcoAccountAnonymousID(useruuid);
		Assert.assertNotNull(anonymousid);

		Assert.assertTrue(useruuid.contentEquals(ecoaccountdao.getEcoAccountID(TestUser.Username, TestUser.Email)));

		Assert.assertTrue(useruuid.contentEquals(ecoaccountdao.getEcoAccountIDFromAnonymousID(anonymousid)));

		Assert.assertTrue(
				useruuid.contentEquals(ecoaccountdao.getEcoAccountIDFromRecycleBinUserID(TestUser.recyclebinuserid)));

		Assert.assertTrue(TestUser.recyclebinuserid == ecoaccountdao.getEcoAccountRecycleBinUserID(useruuid));
	}

	@Test
	public void itEcoAccountManipulate() {
		final String useruuid = AccessControlManager.getUsernameUUID(TestUser.Username);

		{
			final ProductType itemtype = new ProductType(847130, 303, "Laptops/tablets");
			final IndividualProductData item = new IndividualProductData(itemtype, "04047111123453", "Some Product ID",
					"Some Serial Number", "Laptop example", DataConversion.readDate("2010-12-24"));

			// Insert recycled items
			int count = 0;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-01"), 2, 7, WEEEsEndOfLifeState.WORKING, item));
			++count;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-02"), 2, 7, WEEEsEndOfLifeState.REUSABLE, item));
			++count;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-03"), 2, 7, WEEEsEndOfLifeState.BROKEN, item));
			++count;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-04"), 2, 7, WEEEsEndOfLifeState.WORKING, itemtype));
			++count;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-05"), 2, 7, WEEEsEndOfLifeState.REUSABLE, itemtype));
			++count;
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithRecycledWEEEItem(useruuid,
					DataConversion.readDate("2018-05-06"), 2, 7, WEEEsEndOfLifeState.BROKEN, itemtype));
			++count;

			final EcoAccountRecord account = ecoaccountdao.getEcoAccount(useruuid);
			Assert.assertNotNull(account);

			// Assert that we got a list of recycled items
			Assert.assertTrue(count == account.getEcoBalance().getRecycled().getItemListElement().size());

			// Check that we got Eco-credits from our recycling activities
			Assert.assertTrue(0.0 < account.getEcoBalance().getEcoCredit().getValue().doubleValue());
		}

		{
			final double ecodebits = 12.0;
			final double ecocredits = 7.5;
			final ArrayList<IndividualProductData> purchases = new ArrayList<IndividualProductData>();
			purchases.add(new IndividualProductData(new ProductType(847130, 303, "Laptops/tablets"), "04047111123453",
					"Some Product ID", "Some Serial Number", "Laptop example", DataConversion.readDate("2010-12-24")));
			purchases.add(new IndividualProductData(new ProductType(23), "43210987654321", "Some Product ID",
					"Some Serial Number", "Food example", DataConversion.readDate("2010-12-24")));
			purchases.add(new IndividualProductData(new ProductType(847130, 303, "Laptops/tablets"), "12345678901234",
					"Some Product ID", "Some Serial Number", "Tablet example", DataConversion.readDate("2010-12-24")));

			// Insert purchases
			Assert.assertTrue(ecoaccountdao.updateEcoAccountWithPurchases(useruuid, purchases, ecodebits, ecocredits));

			final EcoAccountRecord account = ecoaccountdao.getEcoAccount(useruuid);
			Assert.assertNotNull(account);

			// Assert that we got a list of purchased items
			Assert.assertTrue(purchases.size() == account.getEcoBalance().getPurchased().getItemListElement().size());
		}
	}

	@Test
	public void itFeedback() {
		final String useruuid = AccessControlManager.getUsernameUUID(TestUser.Username);

		{
			// Products feedback
			final ArrayList<UserProductFeedback> valid = new ArrayList<UserProductFeedback>();
			valid.add(new UserProductFeedback().withFreetext("Some free text")
					.withProductid(new ProductID("04047111123453")).withUserscore(1).withUseruuid(useruuid));
			valid.add(new UserProductFeedback().withProductid(new ProductID("12345678901234")).withUserscore(5)
					.withUseruuid(useruuid));
			Assert.assertTrue(ecoaccountdao.updateProductsFeedback(valid));

			final ArrayList<UserProductFeedback> invalid = new ArrayList<UserProductFeedback>();
			invalid.add(new UserProductFeedback().withUserscore(1));
			Assert.assertFalse(ecoaccountdao.updateProductsFeedback(invalid));
		}

		{
			// System feedback
			final SystemFeedback[] valid = { // Valid cases
					new SystemFeedback().withFreetext("System feedback blah blah").withUserscore(3)
							.withUseruuid(useruuid), // Full
					new SystemFeedback().withUserscore(3).withUseruuid(useruuid) // Minimal
			};
			for (final SystemFeedback test : valid) {
				Assert.assertTrue(ecoaccountdao.updateSystemFeedback(test));
			}

			final SystemFeedback[] invalid = { // Invalid cases
					new SystemFeedback(), // Empty
					new SystemFeedback().withUserscore(3), // Missing User ID
					new SystemFeedback().withUseruuid(useruuid) // Missing score
			};
			for (final SystemFeedback test : invalid) {
				Assert.assertFalse(ecoaccountdao.updateSystemFeedback(test));
			}
		}
	}

	public static void main(String[] args) {
		final EcoAccountDAOIT dao = new EcoAccountDAOIT();

		try {
			dao.itEcoAccountLocale();
			dao.itEcoAccountAccess();
			dao.itEcoAccountManipulate();
			dao.itFeedback();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
}
