/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.test;

import java.io.IOException;
import java.io.InputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.Set;
import java.util.regex.Pattern;

import org.junit.Assert;
import org.junit.Test;
import org.reflections.Reflections;
import org.reflections.scanners.ResourcesScanner;

import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import gr.iccs.isense.circ4life.dao.CommonDAO;
import gr.iccs.isense.circ4life.dao.ProductMasterDataDAO;
import gr.iccs.isense.circ4life.dao.CommonDAO.EcoAccountRecord;
import gr.iccs.isense.circ4life.dao.CommonDAO.IndividualProductData;
import gr.iccs.isense.circ4life.dao.CommonDAO.ProductRecord;
import gr.iccs.isense.circ4life.dao.CommonDAO.ProductType;
import gr.iccs.isense.circ4life.dao.CommonDAO.SystemFeedback;
import gr.iccs.isense.circ4life.dao.CommonDAO.UserProductFeedback;
import gr.iccs.isense.circ4life.dao.CommonDAO.ProductID;
import gr.iccs.isense.circ4life.json.JacksonJSONSerialization;
import gr.iccs.isense.circ4life.rest.resources.openapi.DataConversion;
import gr.iccs.isense.circ4life.test.IntegrationTestsCommon.TestUser;

public class DataAccessObjectTests {
	private final class ProductIDTest {
		public final Long expectedGTIN;
		public final String expectedSerialNumber;
		public final String test;

		public ProductIDTest(Long expectedGTIN, String expectedSerialNumber, String test) {
			super();
			this.expectedGTIN = expectedGTIN;
			this.expectedSerialNumber = expectedSerialNumber;
			this.test = test;
		}
	}

	private static final String PRODUCT_RESOURCES_PREFIX = "debug.json.products";

	private final Calendar calendar = Calendar.getInstance();
	private final Date today = Date.from(calendar.toInstant());
	private final ProductType prodtype = new ProductType(847130, 303, "Laptops/tablets");
	private final ProductRecord product = new ProductRecord(prodtype, "04047111123453", "Some Product ID");
	private final IndividualProductData indivproduct = new IndividualProductData(prodtype, "04047111123453",
			"Some Product ID", "Some Serial Number", "Laptop example", today);
	private final EcoAccountRecord ecoAccountRecord = new EcoAccountRecord(TestUser.SAMPLEUUID, TestUser.Email,
			TestUser.Firstname, TestUser.Lastname, DataConversion.readDate(TestUser.birthday), TestUser.country,
			TestUser.postalcode, TestUser.street, TestUser.city);
	private final UserProductFeedback userproductfeedback = new UserProductFeedback().withFreetext("Some comment here")
			.withProductid(new ProductID(4047111123453L)).withUserscore(3)
			.withUseruuid("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx");
	private final SystemFeedback systemfeedback = new SystemFeedback().withFreetext("Some comment here")
			.withUserscore(3).withUseruuid("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx");

	@Test
	public void testJSONLDtoString() {
		System.out.println("---- String representations ----");

		// TODO: Add assertions
		System.out.println(product.toString());
		System.out.println(indivproduct.toString());
		System.out.println(ecoAccountRecord.toString());
		System.out.println(userproductfeedback.toString());
		System.out.println(systemfeedback.toString());

		System.out.println("------------------------------");
	}

	@Test
	public void testJSONLDSerialization() throws JsonGenerationException, JsonMappingException, IOException {
		System.out.println("---- JSON serialization ----");

		final JsonFactory messagePackFactory = new JsonFactory();
		messagePackFactory.configure(JsonGenerator.Feature.AUTO_CLOSE_TARGET, false);
		final ObjectMapper objmapper = new ObjectMapper(messagePackFactory);
		// TODO: Add assertions
		System.out.println("-- Compact");
		objmapper.writeValue(System.out, prodtype);
		System.out.println();
		objmapper.writeValue(System.out, product);
		System.out.println();
		objmapper.writeValue(System.out, indivproduct);
		System.out.println();
		objmapper.writeValue(System.out, ecoAccountRecord);
		System.out.println();
		objmapper.writeValue(System.out, userproductfeedback);
		System.out.println();
		objmapper.writeValue(System.out, systemfeedback);
		System.out.println();

		System.out.println();

		System.out.println("-- Pretty");
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, prodtype);
		System.out.println();
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, product);
		System.out.println();
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, indivproduct);
		System.out.println();
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, ecoAccountRecord);
		System.out.println();
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, userproductfeedback);
		System.out.println();
		objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, systemfeedback);
		System.out.println();

		System.out.println();

		System.out.println("-- To String and Back");
		Object obj = null;
		try {
			// ProductType class object cannot be created by default constructor.
			// So the de-serialization from String should throw.
			// This is a design decision in order to avoid the possibility of
			// creating an object of this class without at least the HS key
			JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(prodtype), ProductType.class);
			Assert.assertTrue(false); // should never reach here
		} catch (Exception e) {
			System.err.println("Expected thrown exception: " + e.getMessage());
			Assert.assertTrue(true);
		}
		obj = JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(product),
				ProductRecord.class);
		objmapper.writeValue(System.out, obj);
		System.out.println();
		obj = JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(indivproduct),
				IndividualProductData.class);
		objmapper.writeValue(System.out, obj);
		System.out.println();
		obj = JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(ecoAccountRecord),
				EcoAccountRecord.class);
		objmapper.writeValue(System.out, obj);
		System.out.println();
		obj = JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(userproductfeedback),
				UserProductFeedback.class);
		objmapper.writeValue(System.out, obj);
		System.out.println();
		obj = JacksonJSONSerialization.StringtoJSON(JacksonJSONSerialization.JSONtoString(systemfeedback),
				UserProductFeedback.class);
		objmapper.writeValue(System.out, obj);
		System.out.println();

		System.out.println("------------------------------");
	}

	@Test
	public void testDataConversions() throws Exception {
		System.out.println("---- Data conversions ----");

		// Check Date to JSON property String and back
		System.out.println(indivproduct.getPurchaseDateObject());

		System.out.println("--------------------------");
	}

	@Test
	public void testJSONLDIntegrity() {
		System.out.println("---- Data integrity ----");

		final ProductIDTest goodproductids[] = { // Valid product IDs
				new ProductIDTest(4047111123453L, null, "04047111123453"),
				new ProductIDTest(4047111123453L, "012345678901", "04047111123453.012345678901"),
				new ProductIDTest(12345678L, null, "12345678"), // GTIN-8
				new ProductIDTest(123456789012L, null, "123456789012"), // GTIN-12
				new ProductIDTest(1234567890123L, null, "1234567890123"), // GTIN-13
				new ProductIDTest(12345678901234L, null, "12345678901234"), // GTIN-14
		};
		for (final ProductIDTest test : goodproductids) {
			// check product IDs
			try {
				final ProductID pid = new ProductID(test.test);
				// Should not have thrown
				Assert.assertTrue(true);
				// Check values
				Assert.assertTrue(test.expectedGTIN.longValue() == pid.GTIN14.longValue());
				Assert.assertTrue((null == test.expectedSerialNumber) ? test.expectedSerialNumber == pid.SerialNumber
						: 0 == test.expectedSerialNumber.compareTo(pid.SerialNumber));
				// Check GTIN14 to String formatting
				System.out.println(DataConversion.formatGTIN14(test.expectedGTIN));
				Assert.assertTrue(14 == DataConversion.formatGTIN14(test.expectedGTIN).length());
			} catch (IllegalArgumentException e) {
				System.err.println(e.getMessage());

				// Should not have thrown
				Assert.assertTrue(false);
			}
		}

		final String badproductids[] = { // Invalid product IDs
				null, // Null
				"ABC", // No digits
				"1234567890ABCD", // Mixed digits with characters
				"04047111123453.", // Missing serial number
				"123456789", // GTIN-9 Not existing
				"1234567890", // GTIN-10 Not existing
				"12345678901" // GTIN-11 Not existing
		};
		for (final String pid : badproductids) {
			try {
				System.out.println("Testing malformed product ID " + pid);

				new ProductID(pid);

				// Should have thrown
				Assert.assertTrue(false);
			} catch (IllegalArgumentException e) {
				System.err.println(e.getMessage());

				// Should have thrown
				Assert.assertTrue(true);
			}
		}

		final ProductIDTest goodbarcodes[] = { // Valid barcodes
				new ProductIDTest(4047111123453L, null, "(01)04047111123453"),
				new ProductIDTest(4047111123453L, "012345678901", "(01)04047111123453(21)012345678901"),
				new ProductIDTest(12345678L, null, "(01)00000012345678"), // GTIN-8
				new ProductIDTest(123456789012L, null, "(01)00123456789012"), // GTIN-12
				new ProductIDTest(1234567890123L, null, "(01)01234567890123"), // GTIN-13
				new ProductIDTest(12345678901234L, null, "(01)12345678901234"), // GTIN-14
		};
		for (final ProductIDTest test : goodbarcodes) {
			try {
				final ProductID pid = CommonDAO.readProductBarcode(test.test);
				Assert.assertNotNull(pid);
				// Check values
				Assert.assertTrue(test.expectedGTIN.longValue() == pid.GTIN14.longValue());
				Assert.assertTrue((null == test.expectedSerialNumber) ? test.expectedSerialNumber == pid.SerialNumber
						: 0 == test.expectedSerialNumber.compareTo(pid.SerialNumber));
				// Check GTIN14 to String formatting
				System.out.println(DataConversion.formatGTIN14(test.expectedGTIN));
				Assert.assertTrue(14 == DataConversion.formatGTIN14(test.expectedGTIN).length());
			} catch (IllegalArgumentException e) {
				System.err.println(e.getMessage());

				// Should not have thrown
				Assert.assertTrue(false);
			}
		}

		final String badbarcodes[] = { // Invalid barcodes
				null, // Null
				"Foobar", // Malformed
				"(01)ABC", // No digits
				"(01)1234567890ABCD", // Mixed digits with characters
				"(01)04047111123453(21)", // Missing serial number
				"(01)123456789", // GTIN-9 Not existing
				"(01)1234567890", // GTIN-10 Not existing
				"(01)12345678901", // GTIN-11 Not existing
				"(01)12345678", // GTIN-8 too short
				"(01)123456789012", // GTIN-12 too short
				"(01)1234567890123", // GTIN-13 too short
				"(01)04047111123453(21)012345678901234567890" // Serial Number too long
		};
		for (final String barcode : badbarcodes) {
			System.out.println("Testing malformed barcode " + barcode);
			Assert.assertNull(CommonDAO.readProductBarcode(barcode));
		}

		final int goodscores[] = { 1, 2, 3, 4, 5 };
		for (final int score : goodscores) {
			try {
				new UserProductFeedback().withUserscore(score);
				// Should reach here
				Assert.assertTrue(true);
			} catch (Exception e) {
				// Should NOT reach here
				Assert.assertTrue(false);
			}
		}

		final int badscores[] = { -1, 0, 6 };
		for (final int score : badscores) {
			try {
				new UserProductFeedback().withUserscore(score);
				// Should NOT reach here
				Assert.assertTrue(false);
			} catch (Exception e) {
				// Should reach here
				Assert.assertTrue(true);
			}
		}

		System.out.println("--------------------------");
	}

	@Test
	public void testJSONLDAssignment() {
		// TODO: Implement
	}

	@Test
	public void testJSONLDStreamRead() {
		System.out.println("---- JSON Stream read ----");

		final JsonFactory messagePackFactory = new JsonFactory();
		messagePackFactory.configure(JsonGenerator.Feature.AUTO_CLOSE_TARGET, false);
		final ObjectMapper objmapper = new ObjectMapper(messagePackFactory);

		// Read from JAR resources
		final Reflections reflections = new Reflections(PRODUCT_RESOURCES_PREFIX, new ResourcesScanner());
		final Set<String> resources = reflections.getResources(Pattern.compile(".*\\.json"));
		System.out.println("Found " + resources.size() + " JSON files in " + PRODUCT_RESOURCES_PREFIX);

		for (final String resource : resources) {
			System.out.println("Trying to load " + resource);

			try {
				final InputStream is = ProductMasterDataDAO.class.getResourceAsStream("/" + resource);

				if (null == is) {
					System.err.println("Could not load resource " + resource);
					continue;
				}

				final ProductRecord prodrec = objmapper.readerFor(ProductRecord.class).readValue(is);
				System.out.println("Successfully loaded " + resource);
				objmapper.writerWithDefaultPrettyPrinter().writeValue(System.out, prodrec);
				System.out.println();

				is.close();
			} catch (Exception e) {
				System.err.println("Could not load resource " + resource + ", details:" + e.getMessage());
				continue;
			}
		}

		System.out.println("--------------------------");
	}
}
