/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life;

/**
 * This class implements the Eco-credit calculation for products as described in
 * Deliverable 2.4: Eco-credits method final definition.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class EcoCreditCalculator {
	/**
	 * Calculate the Eco-credits of a recycled product with the specified parameters
	 * according to the formula of Eq. 1 in Deliverable 2.4.
	 *
	 * @param params The parameters that describe the recycled product.
	 * @return The Eco-credits value.
	 * @throws IllegalArgumentException thrown if the specified parameters object is
	 *                                  not valid.
	 */
	public double calculate(final EcoCreditParameters params) throws IllegalArgumentException {
		// Validate the input in params object
		validate(params);

		return params.getWeightOfExergy() * params.getExergy()
				+ params.getWeightParameterB() * params.getEndofLifeState()
				+ params.getWeightParameterC() * params.getLifetimeFactor();
	}

	/**
	 * Validate a {@link EcoCreditParameters} object.
	 *
	 * @param params The {@link EcoCreditParameters} object to validate.
	 * @throws IllegalArgumentException thrown if the specified parameters object is
	 *                                  not valid.
	 */
	private void validate(final EcoCreditParameters params) throws IllegalArgumentException {
		// TODO: Implement parameters validation
	}
}
