/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life;

import java.util.Collection;

import gr.iccs.isense.circ4life.datamodels.MaterialFactors;

/**
 * This class implements the Materials Thermodynamic Rarity calculation for WEEE
 * products as described in Deliverable 2.4: Eco-credits method final
 * definition.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class WEEEsMaterialThermodynamicRarityCalculator {
	/**
	 * A list of Material recyclability and rarity factor pairs.
	 */
	private final Collection<MaterialFactors> material_factors;

	/**
	 * No default constructor, it does not make sense. All parameters should be
	 * provided by user.
	 */
	@SuppressWarnings("unused")
	private WEEEsMaterialThermodynamicRarityCalculator() {
		material_factors = null;
	}

	/**
	 * Constructor.
	 *
	 * @param material_factors a list of Material recyclability and rarity factor
	 *                         pairs.
	 */
	public WEEEsMaterialThermodynamicRarityCalculator(final Collection<MaterialFactors> material_factors) {
		this.material_factors = material_factors;
	}

	/**
	 * Get Materials Thermodynamic Rarity sum (exergy).
	 *
	 * @return the sum of the products of material recyclability by rarity.
	 */
	public double getMaterialsExergy() {
		double exergy_sum = 0.0;

		for (final MaterialFactors me : material_factors) {
			exergy_sum += me.getMass_in_grams() * me.getRarity() * me.getRecyclability();
		}

		return exergy_sum;
	}
}
