/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest;

import java.util.logging.ConsoleHandler;
import java.util.logging.Logger;

import org.glassfish.jersey.jackson.JacksonFeature;
import org.glassfish.jersey.logging.LoggingFeature;
import org.glassfish.jersey.server.ResourceConfig;

import gr.iccs.isense.circ4life.logging.Log;
import gr.iccs.isense.circ4life.rest.config.ConfigProperties;
import gr.iccs.isense.circ4life.rest.resources.Information;
import gr.iccs.isense.circ4life.rest.resources.openapi.Ecocredits;

/**
 * This class implements the Eco-credit calculation for products in the library
 * {@link gr.iccs.isense.circ4life.EcoCreditCalculator} as a RESTful Web Service
 * API.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class EcoCreditCalculatorModule extends ResourceConfig {
	/**
	 * Logger.
	 */
	public final static Logger logger = Log.getConfiguredLogger(EcoCreditCalculatorModule.class);

	/**
	 * REST service error messages.
	 */
	public static final class ErrorMessages {
		public final static String AUTHENTICATION_REQUIRED = "Authentication required.";
		public final static String SESSIONUSERID_FAILURE = "Cannot get user ID from session.";
		public final static String ANONYMOUSID_FAILURE = "Cannot generate user anonymous ID.";
		public final static String UNKNOWN_CALC_ERROR = "Unknown calculation error";
		public final static String MISSING_HSKEY = "Missing Harmonized System code key from parameters";
		public final static String UNKNOWN_HSKEY = "Unknown Harmonized System code key";
		public final static String UNKNOWN_UNUKEY = "Unknown UNU code key";
		public final static String INVALID_LIFETIME_USAGE = "Invalid product lifetime usage period";
		public final static String CANNOT_CREATE_LIFETIMEFACTOR_CALC = "Cannot create lifetime factor calculator";
		public final static String LIFETIMEFACTOR_PARAMS_NOT_FOUND = "No lifetime factor parameters found";
		public final static String CONSTANT_WEIGHTS_NOT_FOUND = "No weight constants found";
		public final static String FOODWASTE_PARAMS_NOT_FOUND = "No food waste parameters found";
		public final static String PRODUCT_PARAMS_NOT_FOUND = "No Eco-credits parameters found for this product";
		public final static String MALFORMED_ECOCREDIT_PARAMS = "Malformed Eco-credists parameters";
	}

	/**
	 * Constructor.
	 */
	public EcoCreditCalculatorModule() {
		// REST resource classes
		register(Information.class);
		register(Ecocredits.class);

		// Jackson JSON serialization
		register(JacksonFeature.class);

		// Setup logging
		logger.setLevel(ConfigProperties.getLoggingLevel(EcoCreditCalculatorModule.class));
		final ConsoleHandler loggerhandler = new ConsoleHandler();
		loggerhandler.setLevel(logger.getLevel());
		logger.addHandler(loggerhandler);
		logger.setUseParentHandlers(false);
		final LoggingFeature.Verbosity verbosity = ConfigProperties
				.getLoggingVerbosity(EcoCreditCalculatorModule.class);
		register(new LoggingFeature(logger, logger.getLevel(), verbosity, LoggingFeature.DEFAULT_MAX_ENTITY_SIZE));

		logger.info("Logger level: " + logger.getLevel() + ", Verbosity: " + verbosity + ", Parent Logger Level: "
				+ logger.getParent().getLevel() + ", using parent logger handlers: " + logger.getUseParentHandlers());
	}
}
