/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.test;

import org.junit.Assert;
import org.junit.Test;
import org.openapitools.model.EcoCreditParameters;
import org.openapitools.model.FoodParameters;
import org.openapitools.model.ProductType;
import org.openapitools.model.WEEEsParameters;

import gr.iccs.isense.circ4life.json.JacksonJSONSerialization;
import gr.iccs.isense.circ4life.rest.resources.validation.POSTDataValidator;

/**
 * Tests for EcoCreditCalculator REST Web service.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class EcoCreditCalculatorWSTests {
	@Test
	public void testPOSTDataValidator() {
		System.out.println("-------------------------------------------------");

		System.out.println("VALID POST DATA");
		final EcoCreditParameters[] valid = { // VALID cases
				// Food waste
				new EcoCreditParameters().producttype(new ProductType().hskey(23))
						.foodparameters(new FoodParameters().chemicalExergy(1.0).mass(2.0).weightD(3.0)),
				// WEEEs
				new EcoCreditParameters().producttype(new ProductType().hskey(847130))
						.weeesparameters(new WEEEsParameters().eolstate("working").monthsofuse(7).weeestype(303)
								.weightA(1.0).weightB(2.0).weightC(3.0).yearsofuse(2))
				// End of tests
		};
		for (final EcoCreditParameters params : valid) {
			System.out.println(JacksonJSONSerialization.JSONtoString(params));
			Assert.assertTrue(POSTDataValidator.isValid(params));
		}

		System.out.println("-------------------------------------------------");

		System.out.println("INVALID POST DATA");
		final EcoCreditParameters[] invalid = { // INVALID cases
				// Empty
				new EcoCreditParameters(),
				// No specific parameters
				new EcoCreditParameters().producttype(new ProductType().hskey(1)),
				// missing food parameters values
				new EcoCreditParameters().producttype(new ProductType().hskey(1)).foodparameters(new FoodParameters()),
				// missing some food parameters values
				new EcoCreditParameters().producttype(new ProductType().hskey(1))
						.foodparameters(new FoodParameters().chemicalExergy(1.0)),
				// missing WEEEs parameters values
				new EcoCreditParameters().producttype(new ProductType().hskey(1))
						.weeesparameters(new WEEEsParameters()),
				// missing WEEEs parameters values 2
				new EcoCreditParameters().producttype(new ProductType().hskey(847130))
						.weeesparameters(new WEEEsParameters().eolstate("working").monthsofuse(7).weeestype(303)
								.weightA(1.0).weightB(2.0)),
				// Malformed EoL state
				new EcoCreditParameters().producttype(new ProductType().hskey(847130))
						.weeesparameters(new WEEEsParameters().eolstate("foobar").monthsofuse(7).weeestype(303)
								.weightA(1.0).weightB(2.0).weightC(3.0).yearsofuse(2))
				// End of tests
		};
		for (final EcoCreditParameters params : invalid) {
			System.out.println(JacksonJSONSerialization.JSONtoString(params));
			Assert.assertFalse(POSTDataValidator.isValid(params));
		}

		System.out.println("-------------------------------------------------");
	}
}
