/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest.resources;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;

import javax.validation.constraints.Size;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import javax.ws.rs.core.Response.Status;

import gr.iccs.isense.circ4life.rest.EndUserModule;

/**
 * REST end-point for Eco-shopping product auxiliary data access.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
@Path("/ecoshopping/media")
public class EcoShoppingMedia {
	/**
	 * Get the local hosted image of a product.
	 * 
	 * @param productid the GTIN14 unique product ID.
	 * @return the image of the product if found.
	 */
	@GET
	@Path("/image/{productid}")
	@Produces({ "image/png", "image/jpg" })
	public Response getProductImage(@PathParam("productid") @Size(min = 14) final String productid) {
		// This endpoint is public and should not apply any access control, we only
		// provide a manually uploaded image

		// Get the product image path
		final File productimagedir = EndUserModule.getProductImagePath(productid);
		if (!productimagedir.exists()) {
			EndUserModule.logger.warning("Uploaded product image directory not found for product " + productid);

			return Response.status(Status.NOT_FOUND).build();
		}

		final FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				if (name.equalsIgnoreCase(EndUserModule.PRODUCT_IMAGE_PREFIX + ".jpg")
						|| name.equalsIgnoreCase(EndUserModule.PRODUCT_IMAGE_PREFIX + ".png")) {
					// matched our predefined filename
					return true;
				}

				return false;
			}
		};

		for (final String imagefile : productimagedir.list(filter)) {
			EndUserModule.logger.fine("Product image file found: " + imagefile);

			final File productimage = new File(productimagedir, imagefile);

			// Open the image and return a stream to the client
			return Response.ok(new StreamingOutput() {
				@Override
				public void write(OutputStream output) throws IOException, WebApplicationException {
					Files.copy(productimage.toPath(), output);
				}
			}).build();
		}

		EndUserModule.logger.warning("Uploaded product image file not found for product " + productid);
		return Response.status(Status.NOT_FOUND).build();
	}
}
