/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest.resources.openapi;

import java.util.ArrayList;
import java.util.Collection;

import javax.validation.constraints.Size;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.core.Response.Status;

import org.openapitools.api.ProductTypesApi;
import org.openapitools.model.ProductType;

import gr.iccs.isense.circ4life.dao.CommonDAO;
import gr.iccs.isense.circ4life.dao.ProductMasterDataDAO;
import gr.iccs.isense.circ4life.rest.RecycleModule;
import gr.iccs.isense.circ4life.rest.RecycleModule.ErrorMessages;
import gr.iccs.isense.circ4life.rest.resources.AccessControlManager;

/**
 * /product_types end-point implementation.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class ProductTypes extends ProductTypesApi {
	/**
	 * Product inormation Data Access Object.
	 */
	private static final ProductMasterDataDAO productdao = new ProductMasterDataDAO();

	/**
	 * Proxy of security context for session management.
	 */
	private final SecurityContext m_securityContext;

	/**
	 * Constructor.
	 * 
	 * @param securityContext proxy of security context for session management.
	 */
	public ProductTypes(@Context SecurityContext securityContext) {
		m_securityContext = securityContext;
	}

	@Override
	public Response productTypesGET(@Size(min = 2, max = 2) String locale) {
		if (AccessControlManager.isEnabled() && !AccessControlManager.isUserAuthenticated(m_securityContext)) {
			RecycleModule.logger.severe(ErrorMessages.AUTHENTICATION_REQUIRED);

			return Response.status(Status.FORBIDDEN.getStatusCode(), ErrorMessages.AUTHENTICATION_REQUIRED).build();
		}

		final Collection<CommonDAO.ProductType> product_types = productdao.getSupportedProductTypes(locale);
		if (0 == product_types.size()) {
			RecycleModule.logger.severe(ErrorMessages.PRODUCT_TYPES_EMPTY);

			return Response.status(Status.NOT_FOUND.getStatusCode(), ErrorMessages.PRODUCT_TYPES_EMPTY).build();
		}

		final ArrayList<ProductType> result = new ArrayList<ProductType>();
		for (final CommonDAO.ProductType e : product_types) {
			final ProductType prodtype = new ProductType();
			result.add(prodtype.hskey(e.HSkey).unukey(e.UNUkey).text(e.Description));
		}

		return Response.ok().entity(result).build();
	}

}
