/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest.resources.validation;

import org.openapitools.model.RecycleEvaluationReport;

import gr.iccs.isense.circ4life.rest.RecycleModule;
import gr.iccs.isense.circ4life.rest.resources.openapi.DataConversion;

/**
 * POST data structures validator.
 * 
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class POSTDataValidator {
	/**
	 * Validate recycle evaluation report.
	 * 
	 * @param recycleEvaluationReport the recycle evaluation report.
	 * @return <code>true</code> if the object is valid, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isValid(final RecycleEvaluationReport recycleEvaluationReport) {
		try {
			// Required properties
			// RecycleEvaluationReport.consumer
			// RecycleEvaluationReport.report
			// RecycleEvaluationReport.report.recycledate
			// RecycleEvaluationReport.report.item
			// RecycleEvaluationReport.report.item.wastetype
			recycleEvaluationReport.getConsumer().longValue();
			DataConversion.readDate(recycleEvaluationReport.getReport().getRecycledate());
			recycleEvaluationReport.getReport().getItem().getWastetype().intValue();

			// If there is a lifetime present, it has these required properties
			// RecycleEvaluationReport.report.lifetime.years
			// RecycleEvaluationReport.report.lifetime.extramonths
			if (null != recycleEvaluationReport.getReport().getLifetime()) {
				recycleEvaluationReport.getReport().getLifetime().getYears().intValue();
				recycleEvaluationReport.getReport().getLifetime().getExtramonths().intValue();
			}

			// if there is a product item present, it needs to have some properties set
			if (null != recycleEvaluationReport.getReport().getItem().getProduct()) {
				recycleEvaluationReport.getReport().getItem().getProduct().getGtin().longValue();
				if (0 == recycleEvaluationReport.getReport().getItem().getProduct().getSerialnum().length()) {
					throw new IllegalArgumentException(
							"Empty serial number found in product item of recycle evaluation report");
				}
				if (0 == recycleEvaluationReport.getReport().getItem().getProduct().getName().length()) {
					throw new IllegalArgumentException("Empty name found in product item of recycle evaluation report");
				}

				DataConversion.readDate(recycleEvaluationReport.getReport().getItem().getProduct().getPurchasedate());
			}

			// All OK
			return true;
		} catch (Exception e) {
			RecycleModule.logger.severe("Recycle item evaluation report validation error:" + e.getMessage());

			return false;
		}
	}
}
