/**
 * Institute of Communication and Computer Systems (ICCS)
 * i-sense Research Group
 */
package gr.iccs.isense.circ4life.rest;

import java.util.logging.ConsoleHandler;
import java.util.logging.Logger;

import org.glassfish.jersey.jackson.JacksonFeature;
import org.glassfish.jersey.logging.LoggingFeature;
import org.glassfish.jersey.server.ResourceConfig;

import gr.iccs.isense.circ4life.logging.Log;
import gr.iccs.isense.circ4life.rest.config.ConfigProperties;
import gr.iccs.isense.circ4life.rest.resources.Information;
import gr.iccs.isense.circ4life.rest.resources.openapi.AnonymousEcoAccount;
import gr.iccs.isense.circ4life.rest.resources.openapi.Product;

/**
 * This class implements the Retailer module integration with the ICT platform
 * as a RESTful Web Service API.
 *
 * @author Miltiadis Koutsokeras, miltos.koutsokeras@iccs.gr
 */
public class RetailerModule extends ResourceConfig {
	/**
	 * Logger.
	 */
	public final static Logger logger = Log.getConfiguredLogger(RetailerModule.class);

	/**
	 * REST service error messages.
	 */
	public static final class ErrorMessages {
		public final static String AUTHENTICATION_REQUIRED = "Authentication required.";
		public final static String ECOACCOUNT_NOTFOUND = "Cannot find Eco-account for anonymous ID.";
		public final static String UNSUPPORTED_BARCODE = "Barcode is not supported.";
		public final static String PRODUCT_NOTFOUND = "Product cannot be found.";
		public final static String PRODUCT_MALFORMED_RECORD = "Product Master Data Record is malformed.";
		public final static String MALFORMED_PURCHASE_LIST = "Malformed consumer purchase list";
		public final static String ECOACCOUNT_UPDATE_FAILURE = "Could not update Eco-account";
		public final static String CONVERSION_ERROR = "Data conversion error";
	}

	/**
	 * Constructor.
	 */
	public RetailerModule() {
		// REST resource classes
		register(Information.class);
		register(AnonymousEcoAccount.class);
		register(Product.class);

		// Jackson JSON serialization
		register(JacksonFeature.class);

		// Setup logging
		logger.setLevel(ConfigProperties.getLoggingLevel(RetailerModule.class));
		final ConsoleHandler loggerhandler = new ConsoleHandler();
		loggerhandler.setLevel(logger.getLevel());
		logger.addHandler(loggerhandler);
		logger.setUseParentHandlers(false);
		final LoggingFeature.Verbosity verbosity = ConfigProperties.getLoggingVerbosity(RetailerModule.class);
		register(new LoggingFeature(logger, logger.getLevel(), verbosity, LoggingFeature.DEFAULT_MAX_ENTITY_SIZE));

		logger.info("Logger level: " + logger.getLevel() + ", Verbosity: " + verbosity + ", Parent Logger Level: "
				+ logger.getParent().getLevel() + ", using parent logger handlers: " + logger.getUseParentHandlers());
	}
}
